/*
    ScriptExec - binary bundled into Platypus-created applications
    Copyright (C) 2006 Sveinbjorn Thordarson <sveinbjornt@simnet.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import <Cocoa/Cocoa.h>
#import <Security/Authorization.h>

#import "STChecksum.h"
#import "SillyStringEncrypt.h"

//limits
#define	kMaxFileArguments		1024
#define	kMaxPathLength			1024

// execution modes
#define	kNormalExecution		0
#define	kPrivilegedExecution	1

// output modes
#define	kNoOutput			1
#define	kProgressBarOutput	2
#define	kTextOutput			3

// path to temp script file
#define	kTempScriptFile		@"/tmp/pltpsTmpSecureScptFile"

@interface ScriptExecController : NSObject
{
    IBOutlet id cancelButton;
    IBOutlet id messageTextField;
    IBOutlet id progressBar;
	IBOutlet id progressWindow;
	
	IBOutlet id textOutputWindow;
	IBOutlet id textOutputCancelButton;
	IBOutlet id textOutputTextField;
	IBOutlet id textOutputProgressIndicator;
	
	//menu items
	IBOutlet id hideMenuItem;
	IBOutlet id quitMenuItem;
	IBOutlet id aboutMenuItem;
	
	NSTask			*task;

	NSPipe			*outputPipe;
	NSFileHandle	*readHandle;
	NSString		*outputStr;

	NSMutableArray  *arguments;
	NSMutableArray  *fileArgs;

	NSString		*interpreter;
	NSString		*scriptPath;
	NSString		*hiddenScriptPath;
	NSString		*appName;

	int			execStyle;
	int			outputType;
	int			isDroppable;
	int			remainRunning;
	int			encryptAndChecksum;
	int			childPid;
	
	BOOL		isTaskDone;
	BOOL		isTaskStarted;
	
	AuthorizationRef 		authorizationRef;
}
- (void)executeScript;
- (void)checkTaskStatus;
- (void)executeScriptWithPrivileges;
- (void)checkPrivilegedTaskStatus;
- (void)taskFinished;
- (void) getData: (NSNotification *)aNotification;
- (void)loadSettings;
- (IBAction)cancel:(id)sender;
- (void)fatalAlert: (NSString *)message subText: (NSString *)subtext;
@end
